@@ -51,6 +51,7 @@
 #include "ev-document-print.h"
 #include "ev-document-annotations.h"
 #include "ev-document-attachments.h"
+#include "ev-document-text.h"
 #include "ev-selection.h"
 #include "ev-transition-effect.h"
 #include "ev-attachment.h"
@@ -61,7 +62,7 @@
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
-#if (defined (HAVE_POPPLER_PAGE_RENDER)) && (defined (HAVE_CAIRO_PDF) || defined (HAVE_CAIRO_PS))
+#if (defined (HAVE_CAIRO_PDF) || defined (HAVE_CAIRO_PS))
 #define HAVE_CAIRO_PRINT
 #endif
 
@@ -72,15 +73,6 @@
 /* license field from Creative Commons schema, http://creativecommons.org/ns */
 #define LICENSE_URI "/x:xmpmeta/rdf:RDF/rdf:Description/cc:license/@rdf:resource"
 
-typedef struct {
-	PdfDocument *document;
-	char *text;
-	GList **pages;
-	guint idle;
-	int start_page;
-	int search_page;
-} PdfDocumentSearch;
-
 typedef struct {
 	EvFileExporterFormat format;
 
@@ -116,7 +108,6 @@ struct _PdfDocument
 	PopplerFontsIter *fonts_iter;
 	int fonts_scanned_pages;
 
-	PdfDocumentSearch *search;
 	PdfPrintContext *print_ctx;
 
 	GList *layers;
@@ -129,15 +120,14 @@ static void pdf_document_document_images_iface_init      (EvDocumentImagesInterf
 static void pdf_document_document_forms_iface_init       (EvDocumentFormsInterface       *iface);
 static void pdf_document_document_fonts_iface_init       (EvDocumentFontsInterface       *iface);
 static void pdf_document_document_layers_iface_init      (EvDocumentLayersInterface      *iface);
-#ifdef HAVE_POPPLER_PAGE_RENDER
 static void pdf_document_document_print_iface_init       (EvDocumentPrintInterface       *iface);
-#endif
 static void pdf_document_document_annotations_iface_init (EvDocumentAnnotationsInterface *iface);
 static void pdf_document_document_attachments_iface_init (EvDocumentAttachmentsInterface *iface);
 static void pdf_document_find_iface_init                 (EvDocumentFindInterface        *iface);
 static void pdf_document_file_exporter_iface_init        (EvFileExporterInterface        *iface);
 static void pdf_selection_iface_init                     (EvSelectionInterface           *iface);
 static void pdf_document_page_transition_iface_init      (EvDocumentTransitionInterface  *iface);
+static void pdf_document_text_iface_init                 (EvDocumentTextInterface        *iface);
 static void pdf_document_thumbnails_get_dimensions       (EvDocumentThumbnails           *document_thumbnails,
 							  EvRenderContext                *rc,
 							  gint                           *width,
@@ -148,7 +138,6 @@ static EvLinkDest *ev_link_dest_from_dest    (PdfDocument       *pdf_document,
 					      PopplerDest       *dest);
 static EvLink     *ev_link_from_action       (PdfDocument       *pdf_document,
 					      PopplerAction     *action);
-static void        pdf_document_search_free  (PdfDocumentSearch *search);
 static void        pdf_print_context_free    (PdfPrintContext   *ctx);
 static gboolean    attachment_save_to_buffer (PopplerAttachment *attachment,
 					      gchar            **buffer,
@@ -171,10 +160,8 @@ EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
 								 pdf_document_document_fonts_iface_init);
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_LAYERS,
 								 pdf_document_document_layers_iface_init);
-#ifdef HAVE_POPPLER_PAGE_RENDER
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_PRINT,
 								 pdf_document_document_print_iface_init);
-#endif
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_ANNOTATIONS,
 								 pdf_document_document_annotations_iface_init);
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_ATTACHMENTS,
@@ -187,29 +174,10 @@ EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
 								 pdf_selection_iface_init);
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_TRANSITION,
 								 pdf_document_page_transition_iface_init);
+				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_TEXT,
+								 pdf_document_text_iface_init);
 			 });
 
-static void
-pdf_document_search_free (PdfDocumentSearch   *search)
-{
-        PdfDocument *pdf_document = search->document;
-	int n_pages;
-	int i;
-
-        if (search->idle != 0)
-                g_source_remove (search->idle);
-
-	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
-	for (i = 0; i < n_pages; i++) {
-		g_list_foreach (search->pages[i], (GFunc) g_free, NULL);
-		g_list_free (search->pages[i]);
-	}
-	g_free (search->pages);
-	
-	g_free (search->text);
-	g_free (search);
-}
-
 static void
 pdf_document_dispose (GObject *object)
 {
@@ -219,11 +187,6 @@ pdf_document_dispose (GObject *object)
 		pdf_print_context_free (pdf_document->print_ctx);
 		pdf_document->print_ctx = NULL;
 	}
-	
-	if (pdf_document->search) {
-		pdf_document_search_free (pdf_document->search);
-		pdf_document->search = NULL;
-	}
 
 	if (pdf_document->document) {
 		g_object_unref (pdf_document->document);
@@ -376,8 +339,6 @@ pdf_page_render (PopplerPage     *page,
 		 EvRenderContext *rc)
 {
 	cairo_surface_t *surface;
-
-#ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
 
 	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
@@ -406,24 +367,8 @@ pdf_page_render (PopplerPage     *page,
 	cairo_paint (cr);
 
 	cairo_destroy (cr);
-#else /* HAVE_POPPLER_PAGE_RENDER */
-	GdkPixbuf *pixbuf;
-	
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-				 FALSE, 8,
-				 width, height);
 
-	poppler_page_render_to_pixbuf (page,
-				       0, 0,
-				       width, height,
-				       rc->scale,
-				       rc->rotation,
-				       pixbuf);
-	surface = ev_document_misc_surface_from_pixbuf (pixbuf);
-	g_object_unref (pixbuf);
-#endif /* HAVE_POPPLER_PAGE_RENDER */
-
-	return surface;	
+	return surface;
 }
 
 static cairo_surface_t *
@@ -853,6 +798,12 @@ pdf_document_get_backend_info (EvDocument *document, EvDocumentBackendInfo *info
 	return TRUE;
 }
 
+static gboolean
+pdf_document_support_synctex (EvDocument *document)
+{
+	return TRUE;
+}
+
 static void
 pdf_document_class_init (PdfDocumentClass *klass)
 {
@@ -870,6 +821,7 @@ pdf_document_class_init (PdfDocumentClass *klass)
 	ev_document_class->render = pdf_document_render;
 	ev_document_class->get_info = pdf_document_get_info;
 	ev_document_class->get_backend_info = pdf_document_get_backend_info;
+	ev_document_class->support_synctex = pdf_document_support_synctex;
 }
 
 /* EvDocumentSecurity */
@@ -1397,7 +1349,6 @@ pdf_document_images_get_image (EvDocumentImages *document_images,
 			       EvImage          *image)
 {
 	GdkPixbuf       *retval = NULL;
-#ifdef HAVE_POPPLER_PAGE_GET_IMAGE
 	PdfDocument     *pdf_document;
 	PopplerPage     *poppler_page;
 	cairo_surface_t *surface;
@@ -1413,7 +1364,7 @@ pdf_document_images_get_image (EvDocumentImages *document_images,
 	}
 
 	g_object_unref (poppler_page);
-#endif
+
 	return retval;
 }
 
@@ -1555,7 +1506,8 @@ pdf_document_find_find_text (EvDocumentFind *document_find,
 	GList *matches, *l;
 	PopplerPage *poppler_page;
 	gdouble height;
-	
+	GList *retval = NULL;
+
 	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
 	g_return_val_if_fail (text != NULL, NULL);
 
@@ -1568,14 +1520,22 @@ pdf_document_find_find_text (EvDocumentFind *document_find,
 	poppler_page_get_size (poppler_page, NULL, &height);
 	for (l = matches; l && l->data; l = g_list_next (l)) {
 		PopplerRectangle *rect = (PopplerRectangle *)l->data;
-		gdouble           tmp;
+		EvRectangle      *ev_rect;
 
-		tmp = rect->y1;
-		rect->y1 = height - rect->y2;
-		rect->y2 = height - tmp;
+		ev_rect = ev_rectangle_new ();
+		ev_rect->x1 = rect->x1;
+		ev_rect->x2 = rect->x2;
+		/* Invert this for X-style coordinates */
+		ev_rect->y1 = height - rect->y2;
+		ev_rect->y2 = height - rect->y1;
+
+		retval = g_list_prepend (retval, ev_rect);
 	}
-	
-	return matches;
+
+	g_list_foreach (matches, (GFunc)poppler_rectangle_free, NULL);
+	g_list_free (matches);
+
+	return g_list_reverse (retval);
 }
 
 static void
@@ -1849,15 +1809,11 @@ pdf_document_file_exporter_get_capabilities (EvFileExporter *exporter)
 		EV_FILE_EXPORTER_CAN_REVERSE |
 		EV_FILE_EXPORTER_CAN_SCALE |
 #ifdef HAVE_CAIRO_PRINT
-#ifdef HAVE_POPPLER_PAGE_RENDER
 		EV_FILE_EXPORTER_CAN_NUMBER_UP |
 #endif
-#endif
 		
 #ifdef HAVE_CAIRO_PDF
-#ifdef HAVE_POPPLER_PAGE_RENDER
 		EV_FILE_EXPORTER_CAN_GENERATE_PDF |
-#endif
 #endif
 		EV_FILE_EXPORTER_CAN_GENERATE_PS);
 }
@@ -1873,7 +1829,6 @@ pdf_document_file_exporter_iface_init (EvFileExporterInterface *iface)
 	iface->get_capabilities = pdf_document_file_exporter_get_capabilities;
 }
 
-#ifdef HAVE_POPPLER_PAGE_RENDER
 /* EvDocumentPrint */
 static void
 pdf_document_print_print_page (EvDocumentPrint *document,
@@ -1890,7 +1845,6 @@ pdf_document_document_print_iface_init (EvDocumentPrintInterface *iface)
 {
 	iface->print_page = pdf_document_print_print_page;
 }
-#endif /* HAVE_POPPLER_PAGE_RENDER */
 
 static void
 pdf_selection_render_selection (EvSelection      *selection,
@@ -1903,6 +1857,8 @@ pdf_selection_render_selection (EvSelection      *selection,
 				GdkColor         *base)
 {
 	PopplerPage *poppler_page;
+	cairo_t *cr;
+	PopplerColor text_color, base_color;
 	double width_points, height_points;
 	gint width, height;
 
@@ -1913,10 +1869,6 @@ pdf_selection_render_selection (EvSelection      *selection,
 	width = (int) ((width_points * rc->scale) + 0.5);
 	height = (int) ((height_points * rc->scale) + 0.5);
 
-#ifdef HAVE_POPPLER_PAGE_RENDER
-	cairo_t *cr;
-	PopplerColor text_color, base_color;
-	
 	text_color.red = text->red;
 	text_color.green = text->green;
 	text_color.blue = text->blue;
@@ -1945,25 +1897,6 @@ pdf_selection_render_selection (EvSelection      *selection,
 				       &text_color,
 				       &base_color);
 	cairo_destroy (cr);
-#else /* HAVE_POPPLER_PAGE_RENDER */
-	GdkPixbuf *pixbuf;
-	
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-				 TRUE, 8,
-				 width, height);
-
-	poppler_page_render_selection_to_pixbuf (poppler_page,
-						 rc->scale, rc->rotation, pixbuf,
-						 (PopplerRectangle *)points,
-						 (PopplerRectangle *)old_points,
-						 (PopplerSelectionStyle)style,
-						 text,
-						 base);
-	if (*surface)
-		cairo_surface_destroy (*surface);
-	*surface = ev_document_misc_surface_from_pixbuf (pixbuf);
-	g_object_unref (pixbuf);
-#endif /* HAVE_POPPLER_PAGE_RENDER */
 }
 
 static gchar *
@@ -2040,21 +1973,33 @@ pdf_selection_get_selection_region (EvSelection     *selection,
 	return retval;
 }
 
+static void
+pdf_selection_iface_init (EvSelectionInterface *iface)
+{
+        iface->render_selection = pdf_selection_render_selection;
+	iface->get_selected_text = pdf_selection_get_selected_text;
+        iface->get_selection_region = pdf_selection_get_selection_region;
+}
+
+
+/* EvDocumentText */
 static GdkRegion *
-pdf_selection_get_selection_map (EvSelection *selection,
-				 EvPage      *page)
+pdf_document_text_get_text_mapping (EvDocumentText *document_text,
+				    EvPage         *page)
 {
 	PopplerPage *poppler_page;
 	PopplerRectangle points;
 	GList *region;
 	GdkRegion *retval;
 
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+
 	poppler_page = POPPLER_PAGE (page->backend_page);
 
 	points.x1 = 0.0;
 	points.y1 = 0.0;
 	poppler_page_get_size (poppler_page, &(points.x2), &(points.y2));
-	
+
 	region = poppler_page_get_selection_region (poppler_page, 1.0,
 						    POPPLER_SELECTION_GLYPH,
 						    &points);
@@ -2064,13 +2009,49 @@ pdf_selection_get_selection_map (EvSelection *selection,
 	return retval;
 }
 
-static void
-pdf_selection_iface_init (EvSelectionInterface *iface)
+static gchar *
+pdf_document_text_get_text (EvDocumentText  *selection,
+			    EvPage          *page)
 {
-        iface->render_selection = pdf_selection_render_selection;
-	iface->get_selected_text = pdf_selection_get_selected_text;
-        iface->get_selection_region = pdf_selection_get_selection_region;
-        iface->get_selection_map = pdf_selection_get_selection_map;
+	PopplerPage *poppler_page;
+	PopplerRectangle r;
+
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+
+	poppler_page = POPPLER_PAGE (page->backend_page);
+
+	r.x1 = 0;
+	r.y1 = 0;
+	poppler_page_get_size (poppler_page, &(r.x2), &(r.y2));
+
+	return poppler_page_get_text (poppler_page,
+				      POPPLER_SELECTION_WORD,
+				      &r);
+}
+
+static gboolean
+pdf_document_text_get_text_layout (EvDocumentText  *selection,
+				   EvPage          *page,
+				   EvRectangle    **areas,
+				   guint           *n_areas)
+{
+	PopplerPage *poppler_page;
+
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+
+	poppler_page = POPPLER_PAGE (page->backend_page);
+
+	return poppler_page_get_text_layout (poppler_page, (PopplerRectangle **)areas, n_areas);
+}
+
+static void
+pdf_document_text_iface_init (EvDocumentTextInterface *iface)
+{
+        iface->get_text_mapping = pdf_document_text_get_text_mapping;
+        iface->get_text = pdf_document_text_get_text;
+#ifdef HAVE_POPPLER_PAGE_GET_TEXT_LAYOUT
+        iface->get_text_layout = pdf_document_text_get_text_layout;
+#endif
 }
 
 /* Page Transitions */
@@ -2552,7 +2533,6 @@ ev_annot_from_poppler_annot (PopplerAnnot *poppler_annot,
 			ev_annot_text->is_open = poppler_annot_text_get_is_open (poppler_text);
 		}
 			break;
-#ifdef HAVE_POPPLER_ANNOT_FILE_ATTACHMENT_GET_ATTACHMENT
 	        case POPPLER_ANNOT_FILE_ATTACHMENT: {
 			PopplerAnnotFileAttachment *poppler_annot_attachment;
 			EvAnnotationAttachment     *ev_annot_attachment;
@@ -2584,7 +2564,6 @@ ev_annot_from_poppler_annot (PopplerAnnot *poppler_annot,
 				g_object_unref (poppler_attachment);
 		}
 			break;
-#endif /* HAVE_POPPLER_ANNOT_FILE_ATTACHMENT_GET_ATTACHMENT */
 	        case POPPLER_ANNOT_LINK:
 	        case POPPLER_ANNOT_WIDGET:
 			/* Ignore link and widgets annots since they are already handled */
