@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -34,10 +34,10 @@ G_BEGIN_DECLS
 
 #define EV_TYPE_SELECTION            (ev_selection_get_type ())
 #define EV_SELECTION(o)		     (G_TYPE_CHECK_INSTANCE_CAST ((o), EV_TYPE_SELECTION, EvSelection))
-#define EV_SELECTION_IFACE(k)	     (G_TYPE_CHECK_CLASS_CAST((k), EV_TYPE_SELECTION, EvSelectionIface))
+#define EV_SELECTION_IFACE(k)	     (G_TYPE_CHECK_CLASS_CAST((k), EV_TYPE_SELECTION, EvSelectionInterface))
 #define EV_IS_SELECTION(o)	     (G_TYPE_CHECK_INSTANCE_TYPE ((o), EV_TYPE_SELECTION))
 #define EV_IS_SELECTION_IFACE(k)     (G_TYPE_CHECK_CLASS_TYPE ((k), EV_TYPE_SELECTION))
-#define EV_SELECTION_GET_IFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE ((inst), EV_TYPE_SELECTION, EvSelectionIface))
+#define EV_SELECTION_GET_IFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE ((inst), EV_TYPE_SELECTION, EvSelectionInterface))
 
 typedef enum {
 	EV_SELECTION_STYLE_GLYPH,
@@ -45,10 +45,10 @@ typedef enum {
 	EV_SELECTION_STYLE_LINE
 } EvSelectionStyle;
 
-typedef struct _EvSelection	  EvSelection;
-typedef struct _EvSelectionIface   EvSelectionIface;
+typedef struct _EvSelection	       EvSelection;
+typedef struct _EvSelectionInterface   EvSelectionInterface;
 
-struct _EvSelectionIface
+struct _EvSelectionInterface
 {
 	GTypeInterface base_iface;
 
@@ -61,11 +61,9 @@ struct _EvSelectionIface
 					      GdkColor         *text,
 					      GdkColor         *base);
 	gchar     * (* get_selected_text)    (EvSelection      *selection,
-					      EvRenderContext  *rc,
+					      EvPage           *page,
 					      EvSelectionStyle  style,
 					      EvRectangle      *points);
-	GdkRegion * (* get_selection_map)    (EvSelection      *selection,
-					      EvRenderContext  *rc);
 	GdkRegion * (* get_selection_region) (EvSelection      *selection,
 					      EvRenderContext  *rc,
 					      EvSelectionStyle  style,
@@ -82,16 +80,14 @@ void       ev_selection_render_selection     (EvSelection      *selection,
 					      GdkColor         *text,
 					      GdkColor         *base);
 gchar     *ev_selection_get_selected_text    (EvSelection      *selection,
-					      EvRenderContext  *rc,
+					      EvPage           *page,
 					      EvSelectionStyle  style,
 					      EvRectangle      *points);
-GdkRegion *ev_selection_get_selection_map    (EvSelection      *selection,
-					      EvRenderContext  *rc);
 GdkRegion *ev_selection_get_selection_region (EvSelection      *selection,
 					      EvRenderContext  *rc,
 					      EvSelectionStyle  style,
 					      EvRectangle      *points);
-				  
+
 G_END_DECLS
 
 #endif
